'Programm ......... : Universelle LED-Steuerung (ohne Master/Slave-Betrieb)
'
'Programminhalt ... : - PWM LED-Steuerung von bis zu
'                       24 LED wei oder
'                        8 LED farbig (RGB)
'                     - Funktionen:
'                       > mehrere Betriebsarten (LED-Muster)
' /// Achtung /// In dieser Version ist keine TWI-Buskopplung vorgesehen. Hin-
'                 weise in verschiedenen Programmzeilen sind nicht relevant.
'
'Programmierer .... : Ingolf Bauer (ingolf.bauer@nexgo.de)
'Version .......... : 1.0.73
'Datum ............ : 17.08.2012
'
'-------------------------------------------------------------------------------
'
'Simulator: Einstellmglichkeit
' Der folgende Befehl teilt dem Compiler mit, keine Wartezeitschleifen bei den
' Befehlen "Wait" und "Waitms" zu beachten. Dies ermglicht eine schnellere
' Simulation.

'$sim                                              'Wartezeiten aus

' /// Achtung ///
' Unbedingt <$sim> auskommentieren, bevor der AVR programmiert wird, da sonst
' der AVR nicht korrekt arbeitet.
'
'-------------------------------------------------------------------------------
'
'Versionsbersicht
'-----------------
'
'
'
'
'
'V 1.0.73  17.08.2012 - Bearbeitung . : - berarbeitung der Kommentare/Inhalt
'V 1.0.74  16.08.2012 - Testversion . : - kompletter Test aller LED-Muster
'V 1.0.90  05.08.2012 - Information . : - Beginn der Softwareentwicklung
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

'Programmspeicher des MCs (Programm-Flashspeicher): V 1.0.73 - 43%
'
'-------------------------------------------------------------------------------
'
'Informationen zur Software
'--------------------------
' Inhalt                                                                | Zeile
'-------------------------------------------------------------------------------
' 01. Allgemeine Informationen ........................................ |  126
'      Taktfrequenz ................................................... |  129
'      Helligkeitswerte ............................................... |  134
'      Programmierung ................................................. |  151
' 02. Abkrzungsbersicht ............................................. |  158
' 03. Compilereinstellungen ........................................... |  164
' 04. Funktionsbeschreibung ........................................... |  189
'      Allgemeines .................................................... |  192
'      Stromversorgung ................................................ |  197
'      Bedienung, Anzeige und Signalisierung .......................... |  201
' 05. Interrupt ....................................................... |  215
' 06. Einstellung der FUSE- und/oder LOOK-Bits ........................ |  220
' 07. Anschluschema AVR ATmega16 ..................................... |  234
' 08. I/O-Ports ....................................................... |  269
'      Abkrzungen fr die Signalbezeichnung .......................... |  272
'      Portbelegung ................................................... |  296
' 09. Steckverbinder .................................................. |  361
'      Bedienung (parallel zu den Tastern auf der LP) ................. |  367
'      Adressierung (Jumper: Slave-Adressierung) ...................... |  374
'      Reset bei der Programmierung ................................... |  381
'      Reset durch Taster ............................................. |  387
'      Auswahl der LED-Betriebsspannung ............................... |  392
'      Anzeige LED (zweifarbig) ....................................... |  400
'      Einspeisung .................................................... |  406
'      Programmier- und Busanschluss .................................. |  412
'      Anschluss LED 1-24 ............................................. |  426
'      Anschluss LED 25-32 (optional) ................................. |  470
' 10. Definition ...................................................... |  484
'      Schaltkreis .................................................... |  487
'      Schnittstelle .................................................. |  492
'      Compilereinstellungen .......................................... |  497
' 11. Deklaration ..................................................... |  504
'      Subrotine (Call) ............................................... |  507
'      Dimensionierung ................................................ |  522
' 12. Konfiguration ................................................... |  584
'      I/O-Port ....................................................... |  587
'      Timer .......................................................... |  615
'      Interrupt ...................................................... |  634
' 13. Alias ........................................................... |  646
'      Eingang (digital) .............................................. |  649
'      Ausgang (digital) .............................................. |  653
' 14. Initialisierung ................................................. |  683
'      Blinken ........................................................ |  688
'      Diverses ....................................................... |  699
'      EEprom-Variablen auslesen ...................................... |  708
'      PWM-Konstante berechnen ........................................ |  714
' 20. Programm ........................................................ |  721
'      Betriebsart .................................................... |  725
'      LED-Steuerung .................................................. |  736
'      Beispielcode fr weitere LED-Muster ............................ |  928
' 30. Interruptroutine ................................................ | 1094
'      Timer 0 ........................................................ | 1098
'      Timer 2 ........................................................ | 1154
'      Zeitzhler fr die automatische Umschaltung der Betriebsart .... | 1163
'      Lebensbit ...................................................... | 1174
' 40. Unterprogramm ................................................... | 1188
'      Grundhelligkeit ................................................ | 1192
'      Muster ......................................................... | 1203
'      Taste .......................................................... | 1218
'      Taste 1 - kurz (x110_xxxx): 96 ................................. | 1239
'      Taste 2 - kurz (x101_xxxx): 80 ................................. | 1250
'      Taste 3 - kurz (x011_xxxx): 48 ................................. | 1255
'      Taste 1 - lang (x110_xxxx): 96 ................................. | 1270
'      Taste 2 - lang (x101_xxxx): 80 ................................. | 1277
'      Taste 3 - lang (x011_xxxx): 48 ................................. | 1282
'      Steuerung: langsames Ein/Aus-Schalten .......................... | 1298
' 50. Unterprogramm-Call .............................................. | 1313
'      Datenbereich auslesen .......................................... | 1317
'      Umrechnung auf nichtlineare PWM-Dauer .......................... | 1342
'      Array_PCM lschen .............................................. | 1381
'      Wartezeit ...................................................... | 1390
' 60. Funktion ........................................................ | 1406
' 70. Datenbereich .................................................... | 1412
'      Beispielcode fr weitere LED-Muster ............................ | 1416
'-------------------------------------------------------------------------------
'
'01. Allgemeine Informationen
'----------------------------
'Taktfrequenz
'------------
' Als Taktfrequenz wir eine Quarzfrequenz von 16,0 MHz verwendet.
' Bei einer anderen Frequenz sind die Timer-Einstellunge anzupassen.
'
'Helligkeitswerte
'----------------
'Stufung (ST) von 1-60 (PWM-Angleichung an das Empfinden des menschlichen Auges)
'
' ST Wert | ST Wert | ST Wert | ST Wert | ST Wert | ST Wert
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'  1-..0  | 11-..8  | 21-.31  | 31-.68  | 41-119  | 51-184
'  2-..0  | 12-.10  | 22-.34  | 32-.72  | 42-124  | 52-191
'  3-..0  | 13-.11  | 23-.37  | 33-.77  | 43-130  | 53-198
'  4-..1  | 14-.13  | 24-.40  | 34-.81  | 44-137  | 54-206
'  5-..1  | 15-.15  | 25-.44  | 35-.86  | 45-143  | 55-214
'  6-..2  | 16-.18  | 26-.47  | 36-.91  | 46-149  | 56-222
'  7-..3  | 17-.20  | 27-.51  | 37-.96  | 47-156  | 57-230
'  8-..4  | 18-.22  | 28-.55  | 38-102  | 48-163  | 58-238
'  9-..5  | 19-.25  | 29-.59  | 39-107  | 49-170  | 59-246
' 10-..7  | 20-.28  | 30-.63  | 40-113  | 50-177  | 60-255

'Programmierung
'--------------
' Die Programmierung des MCs kann extern oder ber die vorhandene Schnittstelle
' erfolgen.
' Fr die Programmierung ist das Modul "USBasp" verwendbar, welches ber eine
' USB-Schnittstelle an einen PC angeschlossen wird. Es kann direkt ber eine,
' in BASCOM integrierte Einstellung angesprochen werden.

'02. Abkrzungsbersicht
'-----------------------
' BA  ... : Betriebsart
' MC .... : Mikrocontroller
' PWM ... : Pulsweitenmodulation

'03. Compilereinstellungen
'-------------------------
' Hinweise fr ATmega8/ATmega16 mit 1024 Byte SRAM
' Sie gelten ggf. auch fr den hier eingesetzten Mikrocontroller.
'
' Quelle:
' //halvar.at/elektronik/kleiner_bascom_avr_kurs/speicher_hwstack_swstack_frame/
'
' HW-Stack    - Empfehlung: 40 Byte bei INT, sonst 10 Byte ausreichend
'  32 Byte fr INT + je GOSUB 2 Byte, wenn bei INT ein GOSUB erfolgt
'  (bei Verschachtelung ... + n x 2 Byte)
'
' SW-Stack    - Empfehlung: 32 Byte (recht komplexe Aufrufe werden abgedeckt)
'   2 Byte fr jede bergebene Variable einer GOSUB-Routine + 2 Byte fr jede
'   LOCAL-Variable innerhalb einer GOSUB
'   (bei Verschachtelung ... n x 2 Byte)
'
' Frame-Space - Empfehlung: 60 Byte sollten alles abdecken
'                           (bei wenig Speicherverkleinern)
' Beispiel:
'  2 x BYTE        =  2 Byte (erster und zweiter Parameter)
'  1 x WORD        =  2 Byte (dritte Parameter)
'  1 x STRING * 10 = 11 Byte (inkl. Abschlussbyte; vierter Parameter)
'  3 x BYTE        =  3 Byte (lokale Variablen)

'04. Funktionsbeschreibung
'-------------------------
'Allgemeines
'-----------
' Die Hardware fr die Baugruppe wurde auf einer durchkontaktierten Leiter-
' platte aufgebaut.

'Stromversorgung
'---------------
' Sie kann fr den Mikrocontroller und die LED getrennt erfolgen.

'Bedienung, Anzeige und Signalisierung
'-------------------------------------
' Die Bedienung kann ber die Taster, die auf der Leiterplatte angebracht sind,
' erfolgen. In dieser Programmversion erfolgt mit Taster 1 die Fortschaltung der
' Betriebsart. Ein lngerer Tastendruck schaltet auf die Betriebsart 1 um.
' Den anderen Tastern ist zur Zeit keine Funktion zugeordnet.
' Fr den Anschluss einer Weihnachtsbaumbeleuchtung wird zyklisch die Betriebs-
' art automatisch fortgeschaltet.
'
' Fr die Anzeige kommt eine Zweifarb-LED zum Einsatz. Die zyklische Programm-
' abarbeitung wird durch das "Lebensbit" angezeigt.
'
' Mchte man eine akustische Signalisierung haben, ist der Piezogeber auf der
' Leiterplatte zu bestcken und programmtechnisch einzubinden.

'05. Interrupt
'-------------
' Timer 0:  8 Bit, Interrupt bei berlauf (interne Takterzeugung: 60 s)
' Timer 2:  8 Bit, Interrupt bei berlauf (interne Takterzeugung: 10 ms)

'06. Einstellung der FUSE- und/oder LOOK-Bits
'--------------------------------------------
' Fr die Einstellung der Fuse- und Lookbits ist der Bauanleitung ein "screen-
' shot" (Programmer: USBasp) beigelegt.
'
' Anpassungen sind bei der Verwendung des USBasp mit einem internen Brennpro-
' gramm von BASCOM mglich (z. B. Lschung des EEproms zusammen mit dem Brennen
' des Programms).
'
' /// Achtung ///
' Beim ATmega16 ist das JTAG(EN) Flag defaultmig auf Enable gesetzt. Dies
' bedeutet, dass das JTAG-Interface (prozenahe Datenabfrage) aktiviert ist.
' Dann kann aber Port C fr keine anderen Aufgaben genutzt werden!

'07. Anschluschema AVR ATmega16
'-------------------------------
'                   -------------------|_|-------------------
' [0]  LED_33 <--- | 1 [PB0, XCK, T0]                [PA0] 40| ---> LED_11  [0]
' [0]  LED_41 <--- | 2 [PB1, T1]                     [PA1] 39| ---> LED_12  [0]
' [0]  LED_42 <--- | 3 [PB2, INT2, AIN0]             [PA2] 38| ---> LED_13  [0]
' [0]  LED_43 <--- | 4 [PB3, OC0, AIN1]              [PA3] 37| ---> LED_21  [0]
' [0]  LED_51 <--- | 5 [PB4, /SS]                    [PA4] 36| ---> LED_22  [0]
' [~] MOSI,52 <--> | 6 [PB5, MOSI]                   [PA5] 35| ---> LED_23  [0]
' [~] MISO,53 <--> | 7 [PB6, MISO]                   [PA6] 34| ---> LED_31  [0]
' [~]  SCK,61 <--> | 8 [PB7, SCK]                    [PA7] 33| ---> LED_32  [0]
' [~]  /RESET ---> | 9 [/Reset]                     [AREF] 32|
'       P5V_D ==== |10 [VCC]                        [AGND] 31| ==== GND
'         GND ==== |11 [GND]                        [AVCC] 30| ==== P5V_A
'          QZ ++++ |12 [XTAL2]                [TOSC2, PC7] 29| ---> PIZ     [0]
'          QZ ++++ |13 [XTAL1]                [TOSC1, PC6] 28| <--- T3,ADR1 [/]
' [0]  LED_62 <--- |14 [PD0, RXD]               [TDI, PC5] 27| <--- T2,ADR0 [/]
' [0]  LED_63 <--- |15 [PD1, TXD]               [TDO, PC4] 26| <--- T1      [1]
' [0]  LED_71 <--- |16 [PD2, INT0]              [TMS, PC3] 25| ---> LED_rt  [0]
' [0]  LED_72 <--- |17 [PD3, INT1]              [TCK, PC2] 24| ---> LED_gn  [0]
' [0]  LED_73 <--- |18 [PD4, OC1B]              [SDA, PC1] 23| #### SDA     [B]
' [0]  LED_81 <--- |19 [PD5, OC1A]              [SCL, PC0] 22| #### SCL     [B]
' [0]  LED_82 <--- |20 [PD6, ICP1]              [OC2, PD7] 21| ---> LED_83  [0]
'                   -----------------------------------------
'
' Werte in []: Einstellungen nach dem Programmstart
'  ~: Impuls   0: aus   1: ein   /: Einstellung
'  A: analog   B: Bussystem

' /// Achtung ///
' Soll nur die Masterbaugruppe aufgebaut werden, kann die Software so gendert
' werden, dass auch an Port C LEDs angeschlossen werden knnen.
' Dann ndern sich die im Anschlussschema verwendeten Signalbezeichnungen der
' Pins 22-29.

'08. I/O-Ports
'-------------
'Abkrzungen fr die Signalbezeichnung
'-------------------------------------
' /Reset  : Resetfunktion
' ADR<n>  : Adressierung (max. 4 TWI-Slaves per Hardwarekodierung)
' LED_    : Anschluss LED
' PIZ     : Anschluss Piezo-Signalgeber
' SDA     : Busanschluss (Daten)
' SLC     : Busanschluss (Takt)
' T_      : Taster
'
' -<n>    : diverse (n: 11-83); entsprechen den LED-Kanlen von 1-24
' _A      : analog
' _D      : digital
' _gn     : grn
' _rt     : rot
'
' P5V_    : Stromversorgung P 5V
' GND     : Stromversorgung Masse
'
' MISO    : Programmieranschlu
' MOSI    : Programmieranschlu
' QZ      : Anschlu Quarz
' SCK     : Programmieranschlu

'Portbelegung
'------------
'           Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' -   : SV  P5V_A ...... | ............. | Stromversorgung analog ....... | 30
' -   : SV  P5V_D ...... | ............. | Stromversorgung digital ...... | 10
' -   : SV  GND ........ | ............. | Stromversorgung .............. | 11
' -   : SV  GND ........ | ............. | Stromversorgung .............. | 31
' -   : PRG /RESET ..... | ............. | Programmierung/Reset des MCs . |  9
' -   : ................ | ............. | .............................. | 32
' -   : TK  Takt ....... | QZ .......... | Quarzanschluss 1 ............. | 12
' -   : TK  Takt ....... | QZ .......... | Quarzanschluss 2 ............. | 13
'
'Port A     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DA  LED ........ | LED_11 ...... | Ansteuerung LED  1 ........... | 40
' - .1: DA  LED ........ | LED_12 ...... | Ansteuerung LED  2 ........... | 39
' - .2: DA  LED ........ | LED_13 ...... | Ansteuerung LED  3 ........... | 38
' - .3: DA  LED ........ | LED_21 ...... | Ansteuerung LED  4 ........... | 37
' - .4: DA  LED ........ | LED_22 ...... | Ansteuerung LED  5 ........... | 36
' - .5: DA  LED ........ | LED_23 ...... | Ansteuerung LED  6 ........... | 35
' - .6: DA  LED ........ | LED_31 ...... | Ansteuerung LED  7 ........... | 34
' - .7: DA  LED ........ | LED_32 ...... | Ansteuerung LED  8 ........... | 33
'
'Port B     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DA  LED ........ | LED_33 ...... | Ansteuerung LED  9 ........... |  1
' - .1: DA  LED ........ | LED_41 ...... | Ansteuerung LED 10 ........... |  2
' - .2: DA  LED ........ | LED_42 ...... | Ansteuerung LED 11 ........... |  3
' - .3: DA  LED ........ | LED_43 ...... | Ansteuerung LED 12 ........... |  4
' - .4: DA  LED ........ | LED_51 ...... | Ansteuerung LED 13 ........... |  5
' - .5: PRG MOSI, LED .. | MOSI, 52 .... | Programmierung, 14 ........... |  6
' - .6: PRG MISO, LED .. | MISO, 53 .... | Programmierung, 15 ........... |  7
' - .7: PRG SCK,  LED .. | SCK,  61 .... | Programmierung, 16 ........... |  8
'
'Port C     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0:  B  Takt  (Bus). | SCL ......... | TWI-Bussystem ................ | 22
' - .1:  B  Daten (Bus). | SDA ......... | TWI-Bussystem ................ | 23
' - .2: DA  Anzeige .... | LED_gn ...... | LED zweifarbig ............... | 24
' - .3: DA  Anzeige .... | LED_rt ...... | LED zweifarbig ............... | 25
' - .4: DE  Taste ...... | T_1 ......... | Eingabe 1 .................... | 26
' - .5: DE  Taste ...... | T_2 ......... | Eingabe 2 .................... | 27
' - .6: DE  Taste ...... | T_3 ......... | Eingabe 3 .................... | 28
' - .7: DA  Signalgeber  | PIZ ......... | Tonausgabe ................... | 29
'
'Port D     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DA  LED ........ | LED_62 ...... | Ansteuerung LED 17 ........... | 14
' - .1: DA  LED ........ | LED_63 ...... | Ansteuerung LED 18 ........... | 15
' - .2: DA  LED ........ | LED_71 ...... | Ansteuerung LED 19 ........... | 16
' - .3: DA  LED ........ | LED_72 ...... | Ansteuerung LED 20 ........... | 17
' - .4: DA  LED ........ | LED_73 ...... | Ansteuerung LED 21 ........... | 18
' - .5: DA  LED ........ | LED_81 ...... | Ansteuerung LED 22 ........... | 19
' - .6: DA  LED ........ | LED_82 ...... | Ansteuerung LED 23 ........... | 20
' - .7: DA  LED ........ | LED_83 .....  | Ansteuerung LED 24 ........... | 21

'Erluterung
' B  : Bussystem
' DA : digitaler Ausgang
' DE : digitaler Eingang
' Kx : Kanal ist frei konfigurierbar
' PRG: Programmierung
' QZ : Quarzanschluss
' SV : Stromversorgung
'
'09. Steckverbinder
'------------------
'Folgende bersicht der Steckverbinder und Klemmen erleichtert den Anschluss
'der Module:

'Bedienung (parallel zu den Tastern auf der LP)
'----------------------------------------------
'X1. 1        GND       Masse
'X1. 2        T1        Taste 1
'X1. 3        T2        Taste 2
'X1. 4        T3        Taste 3

'Adressierung (Jumper: Slave-Adressierung)
'-----------------------------------------
'X2. 1        GND       Masse
'X2. 2        ADR0      Adresse 0 (Taste 2)
'X2. 3        GND       Masse
'X2. 4        ADR1      Adresse 1 (Taste 3)

'Reset bei der Programmierung
'----------------------------
'Jumper: MC-Auswahl
'X3. 1        PRG_/R    Signal vom Programmiergert
'X3. 2        /Reset    Reset-Eingang am MC

'Reset durch Taster
'------------------
'X4. 1        GND       Masse
'X4. 2        /Reset    Reset-Eingang am MC

'Auswahl der LED-Betriebsspannung
'--------------------------------
'Jumper: Wahl der Einspeisung
'X5. 1        ES1       Einspeisung 1 (+5V)
'X5. 2        +Uext     LED-Spannung
'X5. 3        ES2       Einspeisung 2 (+nV)
'X5. 4        +Uext     LED-Spannung

'Anzeige LED (zweifarbig)
'------------------------
'X6. 1        GND       Masse
'X6. 2        Anode1    LED grn
'X6. 3        Anode2    LED rot

'Einspeisung
'-----------
'X10. 1       +5V       Anschluss 1
'X10. 2       +Uext     Anschluss 2
'X10. 3       GND       Masse

'Programmier- und Busanschluss
'-----------------------------
'fr X21 gilt dieselbe Belegung
'X20. 1       MOSI      Programmier-Steuerleitung
'X20. 2       +5V       DC Betriebsspannung (optional Brcke 2)
'X20. 3       -         -
'X20. 4       SCL       Busleitung (Takt)
'X20. 5       /Reset    Programmier-Steuerleitung
'X20. 6       SDA       Busleitung (Daten)
'X20. 7       SCK       Programmier-Steuerleitung
'X20. 8       GND       Masse
'X20. 9       MISO      Programmier-Steuerleitung
'X20.10       GND       Masse

'Anschluss LED 1-24
'------------------
'                              24x wei   8x farbig
'X30. 1       L01_2     Katode LED  1     LED  1 rot
'X30. 2       L02_2     Katode LED  2     LED  1 grn
'X30. 3       L03_2     Katode LED  3     LED  1 blau
'X30. 4       L04_2     Katode LED  4     LED  2 rot
'X30. 5       L05_2     Katode LED  5     LED  2 grn
'X30. 6       L06_2     Katode LED  6     LED  2 blau
'X30. 7       L07_2     Katode LED  7     LED  3 rot
'X30. 8       L08_2     Katode LED  8     LED  3  grn
'X30. 9       L09_2     Katode LED  9     LED  3 blau
'X30.10       L10_2     Katode LED 10     LED  4 rot
'X30.11       L11_2     Katode LED 11     LED  4 grn
'X30.12       L12_2     Katode LED 12     LED  4 blau
'X30.13       L13_2     Katode LED 13     LED  5 rot
'X30.14       L14_2     Katode LED 14     LED  5 grn
'X30.15       L15_2     Katode LED 15     LED  5 blau
'X30.16       L16_2     Katode LED 16     LED  6 rot
'X30.17       L17_2     Katode LED 17     LED  6 grn
'X30.18       L18_2     Katode LED 18     LED  6 blau
'X30.19       L19_2     Katode LED 19     LED  7 rot
'X30.20       L20_2     Katode LED 20     LED  7 grn
'X30.21       L21_2     Katode LED 21     LED  7 blau
'X30.22       L22_2     Katode LED 22     LED  8 rot
'X30.23       L23_2     Katode LED 23     LED  8 grn
'X30.24       L24_2     Katode LED 24     LED  8 blau
'X30.25       +Uext     LED-Spannung
'X30.26       GND       Masse
'X30.27       +Uext     LED-Spannung
'X30.28       GND       Masse
'X30.29       +Uext     LED-Spannung
'X30.30       GND       Masse
'X30.31       +Uext     LED-Spannung
'X30.32       GND       Masse
'X30.33       +Uext     LED-Spannung
'X30.34       GND       Masse
'X30.35       +Uext     LED-Spannung
'X30.36       GND       Masse
'X30.37       +Uext     LED-Spannung
'X30.38       GND       Masse
'X30.39       +Uext     LED-Spannung
'X30.40       GND       Masse

'Anschluss LED 25-32 (optional)
'------------------------------
'X40. 1       L25_2     Katode LED 25      LED  9 rot
'X40. 2       L26_2     Katode LED 26      LED  9 grn
'X40. 3       L27_2     Katode LED 27      LED  9 blau
'X40. 4       L28_2     Katode LED 28      LED 10 rot
'X40. 5       L29_2     Katode LED 29      LED 10 grn
'X40. 6       L30_2     Katode LED 30      LED 10 blau
'X40. 7       L31_2     Katode LED 31
'X40. 8       L32_2     Katode LED 32
'X40. 9       +Uext     LED-Spannung
'X40.10       GND       Masse
'
'-------------------------------------------------------------------------------
'
'10. Definition
'--------------
'Schaltkreis
'-----------
   $regfile = "m16adef.dat"                       'Prozessor ....... : ATmega16A
   $crystal = 16000000                            'Quarzfrequenz ... : 16,0 MHz

'Schnittstelle
'-------------
' ggf. fr eine Ausgabe im Simulator-Feld (UART)
' $baud = 9600                                    'Baudrate ........ : 9600
                                                  '9600/19200/38400/57600/115200
'Compilereinstellungen
'---------------------
   $hwstack = 40                                  'Hardware Stack
   $swstack = 32                                  'Software Stack
   $framesize = 60                                'Frame Space
'
'-----------------------------------------------------------------------------
'
'11. Deklaration
'---------------
'Subrotine (Call)
'----------------
'Datenmuster
   Declare Sub C_db(byval D_anz As Byte , Byval D_typ As Byte)

'Umrechnung
   Declare Sub C_pwm(byval P_wert As Byte , Byval P_sel As Byte )
'- P_wert: PWM-Wert fr die Umrechnung
'- P_sel : 0 > alle LED gleichartig steuern

'Warten
   Declare Sub C_wait(byval W_wz As Integer )
'Helligkeitswert
   Declare Sub A_null(byval W_pwm As Integer)

'Dimensionierung
'---------------
'Feld
   Dim Led_pwm(24) As Byte                        'Matrix fr die LED
   Dim Led_lr(24) As Byte                         'Matrix Laufrichtung
   Dim Led_bm(3)as Byte                           'Matrix Bitmuster

   Dim Slv_wert(5)as Byte                         'Array (5)

'Flag
   Dim F_ba As Bit                                'Betriebsart starten
   Dim F_lb As Bit                                'Lebensbit
   Dim F_tas As Bit                               'Tastenabfrage
   Dim F_us As Bit                                'BA-Umschaltung

'Konstante
   Dim K_ba As Byte                               'Anzahl der Betriebsarten
   Dim K_gw As Integer                            'Grundwert der Helligkeit
   Dim K_hst As Byte                              'Anzahl an Helligkeitsstufen
   Dim K_pwm As Single                            'normierte PWM-Konstante

'Variable
   Dim Ba As Byte                                 'Betriebsart
   Dim Ba_old As Byte                             'Betriebsart (bisher)
   Dim Pwm As Byte                                'PWM-Wert

   Dim Taste As Byte                              'Taste
   Dim Taste_old As Byte                          'Taste (Merker)

   Dim Wz As Integer                              'Wartezeit
   Dim Zwc As Single                              'Call-Zwischenwert

   Dim Slv_adr As Byte                            'Slave_Adresse
   Dim T_twi As Byte                              'Wartezeit Datenbertragung

'Zhler
   Dim Hz01 As Byte                               'Hilfszhler 01
   Dim Hz02 As Byte                               'Hilfszhler 02
   Dim Hz03 As Byte                               'Hilfszhler 03
   Dim Hz04 As Byte                               'Hilfszhler 04
   Dim Hz05 As Byte                               'Hilfszhler 05
   Dim Hz_int As Integer                          'Hilfszhler Integer

   Dim Hz_db As Byte                              'Hilfszhler Datenbereich
   Dim Hz_gh As Byte                              'Hilfszhler Grundhelligkeit
   Dim Hz_mu1 As Byte                             'Hilfszhler Muster 1
   Dim Hz_mu2 As Byte                             'Hilfszhler Muster 2
   Dim Hz_mu3 As Byte                             'Hilfszhler Muster 3
   Dim Hz_pwm As Byte                             'Hilfszhler PWM
   Dim Hz_sen As Byte                             'Hilfszhler Senden (TWI)
   Dim Hz_t1 As Byte                              'Hilfszhler Anzeige 1
   Dim Hz_t2 As Byte                              'Hilfszhler Anzeige 2

   Dim Z_t0 As Byte                               'Zeitzhler Timer 2
   Dim Z_lb As Byte                               'Zeitzhler Lebensbit
   Dim Z_tas As Byte                              'Lnge Tastendruck
   Dim Z_baw As Dword                             'Zeit fr BA-Wechsel

'Einstellung fr RND (SEED)
   Dim ___rseed As Word                           'Hilfsvariable
'
'-------------------------------------------------------------------------------
'
'12. Konfiguration
'-----------------
'I/O-Port
'--------
'=Port A
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddra = &B1111_1111                             'PA0-7 Aus
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Porta = &B0000_0000                            'PA0-7 passiv
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'=Port B
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrb = &B1111_1111                             'PB0-7 Aus
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portb = &B0000_0000                            'PB0-7 passiv
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'=Port C
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrc = &B1000_1111                             'PC0-3,7 Aus; PC4-6 Ein
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portc = &B0111_0000                            'PC0-4,7 passiv; PC5-6 aktiv
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'=Port D
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrd = &B1111_1111                             'PD0-7 Aus
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portd = &B0000_0000                            'PD0-7 passiv
'
'-------------------------------------------------------------------------------
'
'Timer
'-----
'Normal-Konfiguration
'Die Timerinterrupts 0/1 werden beim jeweiligen Timerberlauf ausgelst.
'PSC: 1-8-64-256-1024
'                                  PSC   TVE   T         f
'T0 = 1/ Quarz* Teiler (1/ 16 MHz*    8* 120 = 60,00 s; 16.667 Hz)
'T2 = 1/ Quarz* Teiler (1/ 16 MHz* 1024* 155 =  9,92 ms;    100 Hz)

'Timer 0 (8 Bit) voreinstellen auf Wert: 256-TVE
   Config Timer0 = Timer , Prescale = 8           'Konfiguration
      Timer0 = 136                                'Voreinstellung

'Timer 2 (8 Bit) voreinstellen auf Wert: 256-TVE
   Config Timer2 = Timer , Prescale = 1024        'Konfiguration
      Timer2 = 100                                'Voreinstellung
'
'-------------------------------------------------------------------------------
'
'Interrupt
'---------
   Enable Timer0                                  'Timer 0: Interrupt
      On Timer0 Int_tim0                          'Timer 0: berlauf

   Enable Timer2                                  'Timer 2: Interrupt
      On Timer2 Int_tim2                          'Timer 2: berlauf

'Zeitsteuerung einschalten
   Enable Interrupts                              'Interrupts global zulassen
'
'-------------------------------------------------------------------------------
'
'13. Alias
'---------
'Eingang (digital)
'-----------------
'Alias nicht verwendet

'Ausgang (digital)
'-----------------
'Port A
   Led_pa Alias Porta

   Led11 Alias Porta.0 : Led12 Alias Porta.1 : Led13 Alias Porta.2
   Led21 Alias Porta.3 : Led22 Alias Porta.4 : Led23 Alias Porta.5
   Led31 Alias Porta.6 : Led32 Alias Porta.7

'Port B
   Led_pb Alias Portb

   Led33 Alias Portb.0
   Led41 Alias Portb.1 : Led42 Alias Portb.2 : Led43 Alias Portb.3
   Led51 Alias Portb.4 : Led52 Alias Portb.5 : Led53 Alias Portb.6
   Led61 Alias Portb.7

'Port C
   Led_gn Alias Portc.2
   Led_rt Alias Portc.3
   Ton Alias Portc.7

'Port D
   Led_pd Alias Portd

   Led62 Alias Portd.0 : Led63 Alias Portd.1
   Led71 Alias Portd.2 : Led72 Alias Portd.3 : Led73 Alias Portd.4
   Led81 Alias Portd.5 : Led82 Alias Portd.6 : Led83 Alias Portd.7
'
'-------------------------------------------------------------------------------
'
'14. Initialisierung
'-------------------
'Restart bei Zeitberschreitung
M_restart:
'Blinken
'-------
'beim Programmstart bzw. nach einem Reset
'Lebensbit
   F_lb = 0                                       'bei Initialisierung inaktiv

   For Hz01 = 1 To 6
      Led_gn = Not Led_gn                         'LED_gn (3x)
      Waitms 300
   Next Hz01

'Diverses
'--------
'Lebensbit aktivieren
   F_lb = 1

'Einstellungen
   K_gw = 20                                      'Grundwert der Helligkeit
   K_ba = 20                                      'Anzahl der Betriebsarten

'EEprom-Variablen auslesen
'-------------------------
'Betriebsart
   Readeeprom Ba , &H01                           'Betriebsart (1 Byte)
      If Ba > 20 Then Ba = 0                      'Erststart: Wert 0

'PWM-Konstante berechnen
'-----------------------
   K_hst = 100
   K_pwm = 255 / K_hst                            '=255/HST^2
   K_pwm = K_pwm / K_hst
'
'-------------------------------------------------------------------------------
'
'20. Programm
'------------
Do
'Betriebsart
'-----------
'Betriebsartenwechsel
M_start:
   If Ba_old <> Ba Then
      Ba_old = Ba
      Hz01 = 0 : Hz02 = 0 : Hz03 = 0 : Hz04 = 0 : Hz05 = 0
   Else
      Gosub Up_taste
   End If

'LED-Steuerung
'-------------
   Select Case Ba_old
'- (keine Funktion)
      Case 0
         Ba = 1

'Gleichzeitiges langsames Ein- und Ausschalten (BA = Farbe)
'Farben: 1 ws - 2 rt - 3 gn - 4 bl - 5 ge - 6 ma - 7 cy
      Case 1 To 7
         Gosub St_ea                              'Farbe

'spontane Helligkeit und Farbe
      Case 8
         Hz01 = Rnd(24)                           'RND(24) erzeugt 0-23
         Incr Hz01
         Led_pwm(hz01) = 1
         Led_pwm(hz01 + 1) = Led_pwm(hz01 + 1) + 20
         Led_pwm(hz01 + 2) = Led_pwm(hz01 + 2) + 40

         Call C_wait(200)                         'Zeitbasis: 5 ms

'Grundhelligkeit mit Blitz, Farbe wei
      Case 9
         Call A_null(30)
         Hz01 = 3 * Rnd(8)                        'RND(8) erzeugt 0-7
         Incr Hz01
         Led_pwm(hz01) = 255
         Led_pwm(hz01 + 1) = 255
         Led_pwm(hz01 + 2) = 255

         Call C_wait(10)                          'Zeitbasis: 5 ms
         Call A_null(30)
         Call C_wait(200)

'Grundhelligkeit mit Blitz, zufllige Farbe des Blitzes
      Case 10
         Call A_null(5)
         Hz01 = 3 * Rnd(8)                        'RND(8) erzeugt 0-7
         Incr Hz01
         Hz02 = Rnd(7)
         Select Case Hz02
            Case 0
               Led_pwm(hz01) = 255
            Case 1
               Led_pwm(hz01 + 1) = 255
            Case 2
               Led_pwm(hz01 + 2) = 255
            Case 3
               Led_pwm(hz01) = 255
               Led_pwm(hz01 + 1) = 255
            Case 4
               Led_pwm(hz01) = 255
               Led_pwm(hz01 + 2) = 255
            Case 5
               Led_pwm(hz01 + 1) = 255
               Led_pwm(hz01 + 2) = 255
            Case 6
               Led_pwm(hz01) = 255
               Led_pwm(hz01 + 1) = 255
               Led_pwm(hz01 + 2) = 255
         End Select

         Call C_wait(10)                          'Zeitbasis: 5 ms
         Call A_null(5)
         Call C_wait(200)

'fortlaufende Farbe der Grundfarben
      Case 11
         Call A_null(1)
'rot:      1- 2- 3
            Led_pwm(1) = 255
'grn:     4- 5- 6
            Led_pwm(5) = 255
'blau:     7- 8- 9
            Led_pwm(9) = 255
'gelb:    10-11-12
            Led_pwm(10) = 255
            Led_pwm(11) = 255
'magenta: 13-14-15
            Led_pwm(13) = 255
            Led_pwm(15) = 255
'cyan:    16-17-18
            Led_pwm(17) = 255
            Led_pwm(18) = 255
'rot:     19-20-21
            Led_pwm(19) = 255
'wei:    22-23-24
            Led_pwm(22) = 255
            Led_pwm(23) = 255
            Led_pwm(24) = 255

         For Hz02 = 1 To 8
            Hz03 = Led_pwm(24)
            Hz04 = Led_pwm(23)
            Hz05 = Led_pwm(22)
               For Hz01 = 24 To 1 Step -3
                  Led_pwm(hz01) = Led_pwm(hz01 - 3)
                  Led_pwm(hz01 -1) = Led_pwm(hz01 - 4)
                  Led_pwm(hz01 -2) = Led_pwm(hz01 - 5)
               Next Hz01
            Led_pwm(3) = Hz03
            Led_pwm(2) = Hz04
            Led_pwm(1) = Hz05
            Call C_wait(300)
         Next Hz02

'farbliches langsames Einschalten (nacheinander) aller LED mit Farbwechsel
      Case 12
         For Hz02 = 1 To 7
         Call A_null(1)

         For Hz01 = 1 To 24 Step 3
C12_01:
            If Hz02 = 1 Or Hz02 = 4 Or Hz02 = 5 Or Hz02 = 7 Then
               Incr Led_pwm(hz01)
            End If

            If Hz02 = 2 Or Hz02 = 4 Or Hz02 = 6 Or Hz02 = 7 Then
               Incr Led_pwm(hz01 + 1)
            End If

            If Hz02 = 3 Or Hz02 = 5 Or Hz02 = 6 Or Hz02 = 7 Then
               Incr Led_pwm(hz01 + 2)
            End If

            Call C_wait(3)

            If Ba <> Ba_old Then Goto C12_03

            If Led_pwm(hz01) > 250 Or Led_pwm(hz01 + 1) > 250 Or Led_pwm(hz01 + 2) > 250 Then
               Goto C12_02
            Else
               Goto C12_01
            End If
C12_02:
         Next Hz01
         Call C_wait(1000)                        'Zeitbasis: 5 ms
         Next Hz02
'Abbruch bei BA-Wechsel
C12_03:

'unterschiedliches Einschalten (Farbe: RGB)
'das Ausschalten erfolgt langsam ber alle LED
   Case 13
      For Hz01 = 1 To 24
         If Led_pwm(hz01) > 1 Then
            Led_pwm(hz01) = Led_pwm(hz01) - 3     'Ausschalten
         End If
         Call C_wait(1)
      Next Hz01

'Zeitsteuerung
      Incr Hz02
      Call C_wait(10)                             'nur fr Tastenabfrage

      If Hz02 = 10 Then                           'Deklaration: Byte
         Hz02 = 0
'LED-Auswahl
M_neu:
         ___rseed = Timer0                        'RND initialisieren
         Hz01 = Rnd(24)                           'RND(24) erzeugt 0-23
         Incr Hz01
         If Hz01 = Hz03 Then Goto M_neu           'nicht die gleiche LED
         Led_pwm(hz01) = 250
         Hz03 = Hz01
      End If

'Muster 1
'Blinken aller LED gleichzeitig (Farbe: wei)
   Case 14
      If F_ba = 1 Then                            'Durchlauf 1x
         Led_bm(1) = &B0101_0101
         Led_bm(2) = &B0101_0101
         Led_bm(3) = &B0101_0101
         F_ba = 0
      End If

      Led_bm(1) = Not Led_bm(1)
      Led_bm(2) = Not Led_bm(2)
      Led_bm(3) = Not Led_bm(3)

      Gosub Up_muster
      Call C_wait(3000)                           'Zeitbasis: 5 ms



'ab hier neue Muster eintragen
     Case Is > 15
      Ba = 1

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Beispielcode fr weitere LED-Muster
'-----------------------------------
'Muster 3 und 4
'      Case 19 To 20
'Bitmuster links- bzw. rechtslaufend
'         If F_ba = 1 Then                         'Durchlauf 1x
'            Led_bm(1) = &B0001_0001
'            Led_bm(2) = &B0001_0001
'            Led_bm(3) = &B0001_0001
'            F_ba = 0
'         End If

'         If Ba = 9 Then
'            Rotate Led_bm(1) , Left
'            Rotate Led_bm(2) , Left
'            Rotate Led_bm(3) , Left
'         Else
'            Rotate Led_bm(1) , Right
'            Rotate Led_bm(2) , Right
'            Rotate Led_bm(3) , Right
'         End If

'         Gosub Up_muster
'            Call C_wait(300)

'Muster 5
'      Case 21
'Zusammenlaufen aller LED
'         If F_ba = 1 Then                         'Durchlauf 1x
'            Call C_db(0 , 2)                      'alle LED-Bitmuster: 0
'            Hz01 = 6
'            F_ba = 0
'            Goto M_m52
'         End If

'         If Led_bm(1) < 255 Then
'            Shift Led_bm(1) , Left
'            Incr Led_bm(1)

'            Shift Led_bm(3) , Right
'            Led_bm(3) = Led_bm(3) + 128
'         Else
'            If Led_bm(2) = 0 Then
'               Led_bm(2) = 129
'               Goto M_m51
'            End If

'            If Led_bm(2) < 255 Then
'               Hz02 = Led_bm(2) And &B0000_1111
'               Incr Hz02
'               Led_bm(2) = Led_bm(2) + Hz02
'               Hz02 = 2 ^ Hz01
'               Led_bm(2) = Led_bm(2) + Hz02
'               Decr Hz01
'            End If
'M_m51:
'         End If

'         If Led_bm(2) = 255 Then
'            F_ba = 1
'         End If
'M_m52:
'         Gosub Up_muster
'            Call C_wait(300)

'Laufende LED
'      Case 22
'         Gosub Up_gh                              'Grundhelligkeit einstellen
'         For Hz01 = 1 To 24
'            Call C_db(0 , 2)                      'alle LED-Bitmuster: 0

'            Select Case Hz01
'               Case 1 To 8
'                  Hz02 = Hz01 - 1
'                  Led_bm(1).hz02 = 1
'               Case 9 To 16
'                  Hz02 = Hz01 - 9
'                  Led_bm(2).hz02 = 1
'               Case Else
'                  Hz02 = Hz01 - 17
'                  Led_bm(3).hz02 = 1
'            End Select

'            Gosub Up_muster
'               Call C_wait(100)
'         Next Hz01

'Bitmuster aus dem Datenbereich 1
'      Case 23
'         Restore Dat_1
'         Call C_db(9 , 1)

'Bitmuster aus dem Datenbereich 2
'      Case 24
'         Restore Dat_2
'         Call C_db(12 , 1)

'Bitmuster aus dem Datenbereich 3
'      Case 25
'         Restore Dat_3
'         Call C_db(9 , 1)

'zuflliges RGB-Spektrum fr alle LED
'      Case 26
'         For Hz01 = 1 To 22 Step 3
'            ___rseed = Timer0 * Hz01              'RND initialisieren
'            Led_pwm(hz01) = Rnd(240)
'            Led_pwm(hz01 + 1) = Rnd(240)
'            Led_pwm(hz01 + 2) = Rnd(240)
'         Next Hz01
'         Call C_wait(1000)

'RGB-Spektrum fr alle LED
'      Case 27
'/// Achtung ///
'Geschachtelte For-Next-Schleifen sind nicht mglich.
'M_c141:
'         If Hz01 < 255 Then
'            Incr Hz01
'            Goto M_c142
'         End If

'         If Hz02 < 255 Then
'            Incr Hz02
'            Goto M_c142
'         End If

'         If Hz03 < 255 Then
'            Incr Hz03
'            Goto M_c142
'         Else
'            Hz01 = 0 : Hz02 = 0 : Hz03 = 0
'            Incr Hz05
'            If Hz05 = 3 Then Hz05 = 0
'            Goto M_c141
'         End If

'M_c142:
'         For Hz04 = 1 To 22 Step 3
'            Select Case Hz05
'               Case 0
'                  Led_pwm(hz04) = Hz01
'                  Led_pwm(hz04 + 1) = Hz02
'                  Led_pwm(hz04 + 2) = Hz03
'               Case 1
'                  Led_pwm(hz04) = Hz02
'                  Led_pwm(hz04 + 1) = Hz03
'                  Led_pwm(hz04 + 2) = Hz01
'               Case 2
'                  Led_pwm(hz04) = Hz03
'                  Led_pwm(hz04 + 1) = Hz01
'                  Led_pwm(hz04 + 2) = Hz02
'            End Select
'         Next Hz04

'         Call C_wait(10)
   End Select

'Ende Programmschleife
Loop

'Ende Programm
End

'===============================================================================

'-------------------------------------------------------------------------------
'30. === I N T E R R U P T R O U T I N E =======================================
'-------------------------------------------------------------------------------

'Timer 0
'-------
Int_tim0:
Disable Timer2
'Interrupt: Timer 0 (8 Bit), alle 60 s
'Voreinstellung des Zhlers
   Timer0 = 136                                   'Voreinstellung

   Incr Z_t0                                      'Zeitzhler erhhen

   If Z_t0 = 0 Then
'LED einschalten
      Led_pa = 255                                'Port A
      Led_pb = 255                                'Port B
      Led_pd = 255                                'Port D
   End If

'Wenn die PWM-Zeitdauer erreicht ist, wird die LED ausgeschaltet
' /// Achtung ///
' Den folgenden Code nicht als Schleife programmieren, da dann die Bearbeitung
' zu lange dauert.
   If Z_t0 >= Led_pwm(1) Then Led11 = 0
   If Z_t0 >= Led_pwm(2) Then Led12 = 0
   If Z_t0 >= Led_pwm(3) Then Led13 = 0

   If Z_t0 >= Led_pwm(4) Then Led21 = 0
   If Z_t0 >= Led_pwm(5) Then Led22 = 0
   If Z_t0 >= Led_pwm(6) Then Led23 = 0

   If Z_t0 >= Led_pwm(7) Then Led31 = 0
   If Z_t0 >= Led_pwm(8) Then Led32 = 0
   If Z_t0 >= Led_pwm(9) Then Led33 = 0

   If Z_t0 >= Led_pwm(10) Then Led41 = 0
   If Z_t0 >= Led_pwm(11) Then Led42 = 0
   If Z_t0 >= Led_pwm(12) Then Led43 = 0

   If Z_t0 >= Led_pwm(13) Then Led51 = 0
   If Z_t0 >= Led_pwm(14) Then Led52 = 0
   If Z_t0 >= Led_pwm(15) Then Led53 = 0

   If Z_t0 >= Led_pwm(16) Then Led61 = 0
   If Z_t0 >= Led_pwm(17) Then Led62 = 0
   If Z_t0 >= Led_pwm(18) Then Led63 = 0

   If Z_t0 >= Led_pwm(19) Then Led71 = 0
   If Z_t0 >= Led_pwm(20) Then Led72 = 0
   If Z_t0 >= Led_pwm(21) Then Led73 = 0

   If Z_t0 >= Led_pwm(22) Then Led81 = 0
   If Z_t0 >= Led_pwm(23) Then Led82 = 0
   If Z_t0 >= Led_pwm(24) Then Led83 = 0
M_t0:
Enable Timer2
Return

'Timer 2
'-------
Int_tim2:
'Interrupt: Timer 2 (8 Bit), alle 10 ms
   Timer2 = 100                                   'Voreinstellung

'Tastaturabfrage aktivieren
   F_tas = 1                                      'Flag setzen

'Zeitzhler fr die automatische Umschaltung der Betriebsart
'-----------------------------------------------------------
   Incr Z_baw                                     'Zeitzhler; Test: 4.500
   If Z_baw > 90000 Then                          '90.000: 15 Minuten
      Z_baw = 0                                   'Zeitzhler rcksetzen
      Incr Ba
      If Ba > K_ba Then                           'max. BA-Anzahl
         K_ba = 1                                 'BA rcksetzen
      End If
   End If

'Lebensbit
'---------
   Incr Z_lb                                      'Zeitzhler Lebensbit

   If F_lb = 1 Then                               'Flag Lebensbit
      Select Case Z_lb
         Case 0
            Led_gn = 1                            'LED_gn Ein
         Case 5
            Led_gn = 0                            'LED_gn Aus
      End Select
   End If
Return

'-------------------------------------------------------------------------------
'40. === U N T E R P R O G R A M M =============================================
'-------------------------------------------------------------------------------

'Grundhelligkeit
'---------------
Up_gh:
   If F_ba = 1 Then                               'Durchlauf 1x
      For Hz_gh = 1 To K_gw                       'Helligkeit
         Call C_pwm(hz_gh , 1)
      Next Hz_gh
      F_ba = 0
   End If
Return

'Muster
'------
Up_muster:
   Hz_mu3 = 1
   For Hz_mu1 = 1 To 3
      For Hz_mu2 = 0 To 7
         Led_pwm(hz_mu3) = K_gw                   'Grundwert der Helligkeit
         If Led_bm(hz_mu1).hz_mu2 = 1 Then
            Led_pwm(hz_mu3) = 180
         End If
         Incr Hz_mu3
         Next Hz_mu2
   Next Hz_mu1
Return

'Taste
'-----
Up_taste:
' Prfung, ob ein kurzer Druck oder die Taste lnger gehalten wird
   Taste_old = Taste                              'gedrckte Taste merken
   Taste = Pinc And &B0111_0000                   'Kanal C: 4-7 abfragen

   If Taste = 96 Or Taste = 80 Or Taste = 48 Then
      Led_gn = 0
      Led_rt = 1                                  'Anzeige ein
      If Z_tas < 250 Then
         Incr Z_tas                               'Zeitzhler
      End If                                      'bei einer gedrckten Taste
   End If

'Taste wurde nach einer kurzen Bettigung losgelassen
'++++++++++++++++++++++++++++++++++++++++++++++++++++
   If Taste <> Taste_old And Z_tas > 2 And Z_tas <= 150 Then
      Led_rt = 0                                  'Anzeige aus

      Select Case Taste_old                       'gedrckte Taste
'Taste 1 - kurz (x110_xxxx): 96
'------------------------------
'Betriebsart einstellen
         Case 96
            If Ba < K_ba Then                     'max. Anzahl der BA
               Incr Ba
               Writeeeprom Ba , &H01              'Wert ins EEprom schreiben
            Else
               Ba = 1
            End If

'Taste 2 - kurz (x101_xxxx): 80
'------------------------------
'+ + + Funktion noch frei definierbar + + +
         Case 80

'Taste 3 - kurz (x011_xxxx): 48
'------------------------------
'+ + + Funktion noch frei definierbar + + +
         Case 48

      End Select
   End If

'Taste gedrckt bei bereits lngere Bettigungszeit
'++++++++++++++++++++++++++++++++++++++++++++++++++
   If Taste = Taste_old And Z_tas > 150 Then
'18.03.2012
      Led_rt = 0                                  'Anzeige aus

      Select Case Taste_old                       'gedrckte Taste
'Taste 1 - lang (x110_xxxx): 96
'------------------------------
'Betriebsart 1 einstellen
         Case 96
            Ba = 1
            Writeeeprom Ba , &H01                 'Wert ins EEprom schreiben

'Taste 2 - lang (x101_xxxx): 80
'------------------------------
'+ + + Funktion noch frei definierbar + + +
         Case 80

'Taste 3 - lang (x011_xxxx): 48
'------------------------------
'+ + + Funktion noch frei definierbar + + +
         Case 48

      End Select
   End If

'Zeitzhler fr die Lnge der Bettigungszeit zurcksetzen
   If Taste <> Taste_old Then
      Z_tas = 0
   End If

M_uptas:
Return

'Steuerung: langsames Ein/Aus-Schalten
'-------------------------------------
St_ea:
'Einschalten
            For Hz01 = 1 To 100
               Call C_pwm(hz01 , Ba_old)
            Next Hz01
            Call C_wait(2000)                     'Zeitbasis: 5 ms
'Ausschalten
            For Hz02 = 100 To 1 Step -1
               Call C_pwm(hz02 , Ba_old)
            Next Hz02
            Call C_wait(100)                      'Zeitbasis: 5 ms
  Return

'-------------------------------------------------------------------------------
'50. === U N T E R P R O G R A M M - C A L L ===================================
'-------------------------------------------------------------------------------

'Datenbereich auslesen
'---------------------
Sub C_db(d_anz , D_typ)
   If D_typ = 2 Then
      For Hz_db = 1 To 3
         Led_bm(hz_db) = D_anz
      Next Hz_db
      Exit Sub
   End If

   For Hz_db = 1 To D_anz                         'Bitmuster auslesen
      If D_typ = 1 Then
         Read Led_bm(1)
         Led_bm(2) = Led_bm(1) : Led_bm(3) = Led_bm(1)
      Else
'Auslesen von Datenmustern (fr jedes Byte unterschiedlich)
         Read Led_bm(1) : Read Led_bm(2) : Read Led_bm(3)
      End If

      Gosub Up_muster
      Call C_wait(100)

   Next Hz_db
End Sub

'Umrechnung auf nichtlineare PWM-Dauer
'-------------------------------------
Sub C_pwm(p_wert , P_sel)
   Zwc = P_wert * P_wert
   Zwc = Zwc * K_pwm
   Pwm = Int(zwc)

'LED: alle gleiche Farbe und Helligkeit
'- die Farbe wird ber die Betriebsart eingestellt
Disable Timer0
   Call A_null(1)                                 'alle Werte= 1

   For Hz_pwm = 1 To 24 Step 3
'LED rot
      If P_sel = 1 Or P_sel = 2 Or P_sel = 5 Or P_sel = 6 Then
         Led_pwm(hz_pwm) = Pwm
     End If
'LED grn
      If P_sel = 1 Or P_sel = 3 Or P_sel = 7 Then
         Led_pwm(hz_pwm + 1) = Pwm
     End If
'LED blau
      If P_sel = 1 Or P_sel = 4 Or P_sel = 7 Then
         Led_pwm(hz_pwm + 2) = Pwm
      End If
'LED grn_sonder
      If P_sel = 5 Then
         Led_pwm(hz_pwm + 1) = Pwm / 8            '/8: Farbkorrektur
      End If
'LED blau_sonder
      If P_sel = 6 Then
         Led_pwm(hz_pwm + 2) = Pwm / 5            '/5: Farbkorrektur
      End If
   Next Hz_pwm

Enable Timer0
   Call C_wait(8)                                 'Zeitbasis: 5 ms
End Sub

'Array_PCM lschen
'-----------------
'Wert 1: Array lschen
Sub A_null(w_pwm)
   For Hz_pwm = 1 To 24
      Led_pwm(hz_pwm) = W_pwm
   Next Hz_pwm
End Sub

'Wartezeit
'---------
Sub C_wait(w_wz )
   Wz = W_wz
   While Wz > 0
'Tastenabfrage
'Der Aufruf wird ber einen Timer-Interrupt gesteuert.
      If F_tas = 1 Then                           'Flag abfragen
         Gosub Up_taste
         F_tas = 0                                'Reset Flag
      End If
      Waitms 5
      Decr Wz
   Wend
End Sub

'-------------------------------------------------------------------------------
'60. === F U N K T I O N =======================================================
'-------------------------------------------------------------------------------

' (keine)

'-------------------------------------------------------------------------------
'70. === D A T E N B E R E I C H ===============================================
'-------------------------------------------------------------------------------

'Beispielcode fr weitere LED-Muster
'-----------------------------------
'Dat_1:                                            'Anzahl:  9
'   Data &B0000_0000
'   Data &B0000_0001
'   Data &B0000_0010
'   Data &B0000_0100
'   Data &B0000_1000

'   Data &B0001_0000
'   Data &B0010_0000
'   Data &B0100_0000
'   Data &B1000_0000

'Dat_2:                                            'Anzahl: 12
'   Data &B0000_0000
'   Data &B0000_0001
'   Data &B0000_0101
'   Data &B0001_0101
'   Data &B0101_0101

'   Data &B1010_1010
'   Data &B0101_0100
'   Data &B1010_1000
'   Data &B0101_0000
'   Data &B1010_0000

'   Data &B0100_0000
'   Data &B1000_0000

'Dat_3:                                            'Anzahl: 9
'   Data &B0000_0000
'   Data &B1000_0001
'   Data &B1100_0011
'   Data &B1110_0111
'   Data &B1111_1111

'   Data &B1110_0111
'   Data &B1100_0011
'   Data &B1000_0001
'   Data &B0000_0000

'Ende

'###############################################################################